/* KD.C -- Kanji driver for use with a terminal emulation program.
 *
 * KD.EXE is specifically written to turn the UNMODIFIED MS-KERMIT
 *	terminal emulation program into a Japanese Kanji terminal.
 *	It does this by intercepting output requests to the screen
 *	at the DOS interrupt level, and applying its own Kanji
 *	escape sequence interpretation.  It also tries to be a
 *	VT100-like terminal in normal ASCII mode.
 * ------------------------------------------------------------------------
 * Written to be compiled with Turbo C 2.0 (or later presumably) by
 *	Version 1.00,  3-3-90
 *	Version 0.98,  2-16-90
 *	Izumi Ohzawa, 1-8-90
 * 	Group in Neurobiology/School of Optometry
 * 	University of California, Berkeley
 * 	Berkeley, CA 94720
 * 	izumi@violet.berkeley.edu
 *
 *	This program/code is hereby released into the public domain.
 *	No copyright is claimed, and no restriction is imposed for any use,
 *	commercial or otherwise.  Use at your own risk.
 * ------------------------------------------------------------------------
 * Extensively based on "intercep.c" written 7/31/1987 by:
 *	 Ned Konz
 *	 210 Oleeta St.
 *	 Ormond Bch, FL 32074 (904)672-2431
 *	 BIX:nkonz  CIS:76046,223
 *
 * Program INTERCEP was released into the public domain by Ned Konz.
 * ------------------------------------------------------------------------ */
/*
#define DEBUG 1
*/
#include <dos.h>
#include <stdio.h>
#include <mem.h>
#include <alloc.h>
#include <process.h>
#include <io.h>
#include <string.h>
#include <dir.h>
#include "kd.h"
#include "prototyp.h"

#ifdef DEBUG
FILE *ffp;
#endif

char *usage = 
"Program [args...]\n"
"	where \"Program\" usually is a terminal emulation program\n"
"	such as MS-Kermit.  All screen output from \"Program\" must\n"
"	be done via DOS calls.\n";

char *logo = 
"KD - Kanji Driver (version 1.00) by Izumi Ohzawa [compiled: " __DATE__
"]\n"
"\n";

char switchar,	/* DOS parameter switch char (from int 0x21, fn 0x3700) */
	sepchar = '\\';		/* DOS filename separator */


/* === Locally defined function prototypes */

void interrupt inthandler ( Regpack r, Intpack i );
int get_swi_list(unsigned);
void install(void);
void uninstall(void);

/* the 8086/8088 INT instruction */
#define SWI_INSTRUCTION	0xCD

/* int_table[] --
 * table containg numbers of interrupts we're catching
 * and their old handlers
 * should be sorted by most common interrupts first.
 * NO HARDWARE INTERRUPTS!!!
 */
Intblock
int_table[] = {
	 	{0x10, NULL}, 	/* video BIOS interrupt */
		{0x21, NULL},	/* DOS function call */
                {0x28, NULL},   /* DOS internal fast putchar */
		{-1, NULL}	/* terminator */
	      };


/* ####################################################################### */
/* Our single interrupt handler 					   */
/* This function is RE-ENTERED if "Level2Font File" is in KD.INI,
** because char output (via DOS func 2) trapped here will lead to
** font file access, which of course will use DOS calls AGAIN.
** MS-Kermit seems to work OK, although this function is not re-entrant.
** But other programs will probably fail.  I have tried "kd command",
** and "type filename" there.  This will FAIL!
*/
void interrupt
inthandler ( Regpack r, Intpack i )
{
/* the following variables are declared as static to get them
 * off the caller's stack and to ensure that t[] is where
 * we want it to be: from [BP-01] through [BP-06]
 */
static unsigned char far *caller;
static char far *strptr;
static char	ch;
static unsigned char which_int;
static unsigned char func_code;		/* AH */
static unsigned char al_value;		/* AL */
static int	j;
static unsigned int  temp;
static IFP oldhandler;
static Intblock *ibp;
volatile unsigned t[3];	/* to move stack values down by 3 words into */

	enable();	/* enable interrupt (necessary???) */

	/* point to next instruction */
	caller = (char far *)i.ipcs;

	if (caller[-2] != SWI_INSTRUCTION) {	/* was this a non-SWI? (uh-oh!) */
		uninstall();
		exit(-1);
	}

	which_int = caller[-1];	/* which SWI is this? */
	temp = r.ovl.old[2];		/* AX */
	func_code = temp >>8;		/* AH */
	al_value = temp & 0x00ff;	/* AL */

	/* === Check DOS & BIOS functions to intercept ================= */
	if(which_int == 0x21)	/* ### DOS INT ### */
	{
	    switch(func_code)
	    {
		case 2:		/* DOS putchar */
			Process_One_Input(r.dx & 0x00ff);	/* DL */
			return;		/* IRET */
		case 9:		/* Print string terminated by '$' */
			strptr = MK_FP(r.ds, r.dx);
			while((ch = *strptr++) != '$')
			    Process_One_Input((int)ch);
			return;		/* IRET */
		case 0x40:	/* Write to a file or device */
			if(r.ovl.old[1] != 1)	/* BX = file handle */
			    break;	/* not 'stdout' */
			strptr = MK_FP(r.ds, r.dx);
			temp = r.ovl.old[0];	/* CX = # of bytes to write */
			for(j=0; j<temp; j++)
			    Process_One_Input((int)(*strptr++));
			break;

		default:	/* other DOS functions, CHAIN INT to OLD one */
			break;
	    }  /* end of switch(func_code) */
	}
	else if(which_int == 0x10)	/* ### BIOS INT */
	{
	    switch(func_code)
	    {
		case 0:		/* set video mode */
		case 1:		/* set cursor type */
		case 2:		/* set cursor position */
		case 5:		/* select video page */
		case 6:		/* scroll up */
		case 7:		/* scroll down */
		case 9:		/* write att/char */
		case 10:	/* write char only */
		case 11:	/* set pallete */
		case 12:	/* write pixel */
		case 13:	/* read pixel */
			return;	/* Absorb */

		case 14:	/* BIOS tty putchar */
			Process_One_Input((int)al_value);  /* AL */
			return;		/* IRET */


		case 3:		/* read cursor position */		
		case 4:		/* read light pen pos */
		case 8:		/* read att/char */
		case 15:	/* read current video info */		
		default:	/* if not handled here, chain to real bios */
			break;		/* CHAIN to old BIOS */
	    }  /* end of switch(func_code) for BIOS */
	}  /* end of else if(which_int -- 0x10 ) */

	/* If not caught above, chain to the old interrupt handler */
	/* get old handler value */
	for (ibp=int_table; ibp->intnum>=0 && ibp->intnum!=which_int; ibp++)
		;
	if (ibp->intnum < 0)		/* can't happen... */
		return;			/* but if it does, just return "safely" */
	oldhandler = ibp->oldint;

	/* move all our registers down by 3 words on the stack */
	movedata(_SS, FP_OFF(&r), _SS, FP_OFF(t), sizeof(Regpack));
	/* supply a mock flag value with interrupts masked OFF */
	r.ovl.new.flags = i.flags & ~0x0200;
	/* get the address of the routine to chain to */
	r.ovl.new.ipcs = oldhandler;
	/* bump our frame pointer value down by 3 words --
	 * the stack pointer will be loaded from this new value next.
	 * ............................................................*/
	_BP -= 6;
	return;		/* unstack all registers and do an IRET */
}


/* === install our handler for all the named interrupts */
void
install()
{
	Intblock *ibp;
	IFPP vp;

	for (ibp = int_table; ibp->intnum >= 0; ibp++) {
		vp = (IFPP) MK_FP(0, ibp->intnum*4);
		ibp->oldint = *vp;
		disable();
		*vp = inthandler;
		enable();
	}
}

/* === un-install our handler for all the named interrupts */
void
uninstall()
{
	Intblock *ibp;
	IFPP vp;

	for (ibp = int_table; ibp->intnum >= 0; ibp++) {
		if (ibp->oldint != NULL) {
			vp = (IFPP) MK_FP(0, ibp->intnum*4);
			disable();
			*vp = ibp->oldint;
			enable();
		}
	}
}


void
main(int argc, char *argv[])
{
static int childret;
char ProgDir[128], *strptr;
char *progname;

#ifdef DEBUG
	ffp = fopen("debug.out", "w");
#endif
	/* spit out logo */
	fprintf(stderr, "%s", logo);

	if ((switchar = getswitchar()) != '/')
		sepchar = '/';
	progname = strrchr(argv[0], sepchar) + 1;
	*strchr(progname, '.') = '\0';

	/* process cmdline arguments */
	if (argc < 2) {
		fprintf(stderr, "Usage: %s %s", progname, usage);
		exit(1);
	}

	/* Get the directory where this program's EXE file is located */
	strcpy(ProgDir, argv[0]);
	strptr = ProgDir+strlen(argv[0]);
	while(*strptr != '\\') strptr--;
	*strptr = '\0';

	Read_Ini_File(ProgDir);			/* read configuration file */
        if( Read_kjfont(ProgDir) != 0)
                exit(2);
	if( Initialize_Graphics(ProgDir) != 0)
	{
		Exit_Cleanup();
		exit(3);
	}

	/* install our interrupt handler for each interrupt in the list */
	install();
	printf("##### Loading program \"%s\" #####\n", argv[1]);
	/* now run the process */
	childret = spawnvp(P_WAIT, argv[1], argv+1);
	/* and restore our captured interrupts */
	uninstall();

	Exit_Cleanup();

	/* report on child return value */
	if (childret == -1) {
		fprintf(stderr, "%s: Spawn of \"%s\" failed: %s\n",
			progname, argv[1], strerror(NULL));
		exit(4);
	}
	if (childret != 0)
		fprintf(stderr, "%s: Child process \"%s\" exit value: %d\n",
			progname, argv[1], childret);
        exit(0);
}
