/* kjis.h -- Header file for JIS level-1 & -2 Kanji font module & screen
 *		control module.
 *
 *      Written to be compiled with Turbo C 2.0 (or later presumably) by
 *		Izumi Ohzawa, 1-8-90
 *		Group in Neurobiology/School of Optometry
 *		University of California, Berkeley
 *		Berkeley, CA 94720
 *		izumi@violet.berkeley.edu
 *
 *	This program/code is hereby released into the public domain.
 *	No copyright is claimed, and no restriction is imposed for any use,
 *	commercial or otherwise.  Use at your own risk.
 * ------------------------------------------------------------------------ */

#define SIZECACHE	16		/* JIS-2 font cache size in File mode */
#define PIXPERLINE      17              /* line spacing in pixels */
#define BACKSPACE	8
#define DEL_CHAR	127
#define TAB_CHAR	9
#define CR_CHAR		0x0D
#define ESC_CHAR	27

#define VIDBIOS         0x10            /* Video BIOS interrupt */
#define NEWVIDBIOS      0x68            /* Our redirected Video BIOS */

#define BIOS		1	/* Scroll 1 line via BIOS */
#define MMOVE		2	/* Scroll by memmove for Hercules */
#define SCROLL_UP	6	/* BOIS function for scroll up */
#define SCROLL_DOWN	7	/* scroll down */
#define SCROLL_Y	1	/* arg to Cursor_***(int) func. Allow scroll */
#define SCROLL_N	0	/* do not allow scroll with cursor movement */
#define JIS2_NONE	0	/* do not use JIS-2 font */
#define JIS2_FILE	1	/* access JIS-2 font in file as needed */
#define JIS2_RAM	2	/* JIS-2 font loaded in RAM */

typedef struct {
        unsigned int pix16[16];         /* 16x16 font is 32 bytes */
} Fontpack;

/* ---- #define GLOB_ALLOC only in one of the modules that use this file -- */
#ifdef GLOB_ALLOC

int PixelPerRow = PIXPERLINE;	/* Graphics text line spaceing in # of pixels */
int Xcur =0, Ycur =0;		/* current Column, Row */
int lastxpos, lastypos;		/* last cursor Column, Row [kmori] */
int MaxColumn;			/* Last Column on Screen */
int MaxRow;			/* Last Row on Screen */
int MaxX, MaxY;			/* Bottom right pixel position */
int NumColumn;			/* # of Columns, MaxColumn+1 */
int NumRow;			/* # of Rows, MaxRow+1 */
int ScrollMode = BIOS;		/* Method of scrolling */
int ScrollMult = 1;             /* normally scroll by 1 row, for CGA = 2 */
int DefAscColor = 2;		/* This doesn't change once set */
int AscColor = 2;		/* ASCII color, may change via ESC sequence */
int AscBackColor = 0;		/* ASCII background color */
int KanjiColor = 7;		/* Can't change on the fly, by JIS code restriction */
int BackColor = 0;		/* Background color */
int CursorColor = 12;		/* Color of cursor */
int EmphasisColor = 14;		/* Emphasized ASCII color */
int BoldColor = 12;		/* Bold color */
int RevCharColor = 0;		/* Character color for reverse video */
int RevBGColor = 7;		/* Background color for reverse video */
int SetScrollArea = 1;		/* Allow scroll area to be changed by ESC seq. */
int Level2Font = 0;		/* JIS level-2 font 0=Not used, 1=File, 2=RAM */
int GraphDriver = 0;		/* Graphics adapter type */
int GraphMode;			/* Graphics mode (in each adapter type) */
int ColorOK = 0;		/* OK to use color flag */
int OK2MessBIOS = 0;		/* EGA, VGA */
void *Cursor_Bitmap;		/* use for putimage(), getimage() */

#else

extern int PixelPerRow;		/* Graphics text line spaceing in # of pixels */
extern int Xcur, Ycur;		/* current Column, Row */
extern int lastxpos, lastypos;	/* last cursor Column, Row [kmori] */
extern int MaxColumn;		/* Last Column on Screen */
extern int MaxRow;		/* Last Row on Screen */
extern int MaxX, MaxY;		/* Bottom right pixel position */
extern int NumColumn;		/* # of Columns, MaxColumn+1 */
extern int NumRow;		/* # of Rows, MaxRow+1 */
extern int ScrollMode;		/* Method of scrolling */
extern int ScrollMult;          /* normally scroll by 1 row, for CGA = 2 */
extern int DefAscColor;		/* Default ASCII color */
extern int AscColor;
extern int AscBackColor;	/* ASCII background color */
extern int KanjiColor;
extern int BackColor;		/* Background color */
extern int CursorColor;		/* Color of cursor */
extern int EmphasisColor;	/* Emphasized ASCII color */
extern int BoldColor;		/* Bold color */
extern int RevCharColor;	/* Character color for reverse video */
extern int RevBGColor;		/* Background color for reverse video */
extern int SetScrollArea;	/* Allow scroll area to be changed by ESC seq. */
extern int Level2Font;		/* JIS level-2 font 0=Not used, 1=File, 2=RAM */
extern int GraphDriver;		/* Graphics adapter type */
extern int GraphMode;		/* Graphics mode (in each adapter type) */
extern int ColorOK;
extern int OK2MessBIOS;		/* EGA, VGA */
extern void *Cursor_Bitmap;	/* use for putimage(), getimage() */

#endif		/* of ifdef GLOB_ALLOC */


